<?php
/**
* Main class.
*
* @package WP_Stripe_Bancontact
*/

use Functions\Functions;

class WP_Stripe_Bancontact{
  /**
  * Returs instance
  */
  
  

  public static function get_instance(){
		static $instance = null;
		if( is_null($instance)){
			$instance = new WP_Stripe_Bancontact();
			
		}

		return $instance;
  }
  
  private $connected;
  private $cc_checkout_page;
  private $id_checkout_page;
  private $check_payment_page;
  private $checkout_page;
  private $globalObject;

  /**
 * include php files
 */
  public function includes(){
      require FMWES_PLUGIN_DIR . 'includes/db/class-database.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-webhook-handler.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-products-controller.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-orders-controller.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-coupons-controller.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-connect-controller.php';
      require FMWES_PLUGIN_DIR . 'includes/controllers/class-shipping-controller.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-simple-product.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-checkout.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-confirmation.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-checkout-credit-card.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-checkout-ideal.php';
      require FMWES_PLUGIN_DIR . 'includes/shortcodes/class-donation.php';
      require FMWES_PLUGIN_DIR . 'includes/libraries/class-wp-list-table.php';
      require FMWES_PLUGIN_DIR . 'includes/components/class-wp-sb-list-table-products.php';
      require FMWES_PLUGIN_DIR . 'includes/components/class-wp-sb-list-table-orders.php';
      require FMWES_PLUGIN_DIR . 'includes/components/class-wp-sb-list-table-coupons.php';
      require FMWES_PLUGIN_DIR . 'includes/functions/functions.php';
      require FMWES_PLUGIN_DIR . 'includes/functions/http-functions.php';
      require FMWES_PLUGIN_DIR . 'includes/pages/cart.php';
      require_once  FMWES_PLUGIN_DIR . 'vendor/autoload.php';
  }

  public function register(){
    add_action('plugins_loaded', function(){
      $loaded = load_plugin_textdomain('wp-stripe-bancontact', false, FMWES_PLUGIN_DIRNAME . '/languages');
    });
    add_action('init', function(){
  		if( class_exists ('Database')){
  			$database = new Database();
  			$database->create_tables();
  		}
  	});
    add_action('init', array($this, 'filter_nav_menus'));
    add_action('wp_loaded', function(){
      $this->connected = get_option('fmwes_refresh_token')?true:false;
      $cc_checkout_page_id = get_option('fmwes_cc_checkout_page');
      if($cc_checkout_page_id > 0){
        $this->cc_checkout_page = get_post($cc_checkout_page_id);
      }
      $id_checkout_page_id = get_option('fmwes_id_checkout_page');
      if($id_checkout_page_id > 0){
        $this->id_checkout_page = get_post($id_checkout_page_id);
      }
      $check_payment_page_id = get_option('fmwes_check_payment_page');
      if($check_payment_page_id > 0){
        $this->check_payment_page = get_post($check_payment_page_id);
      }
      $checkout_page_id = get_option('fmwes_checkout_page');
      if($checkout_page_id > 0){
        $this->checkout_page = get_post($checkout_page_id);
      }
      if ($this->connected && isset($this->cc_checkout_page) && isset($this->id_checkout_page) && isset($this->check_payment_page) && isset($this->checkout_page)) {
        $this->globalObject = array(
          'homeUrl' => esc_url(home_url()),
          'nonce' => wp_create_nonce('wp_rest'),
          'loggedin' => is_user_logged_in(),
          'publicKey' => esc_attr(FMWES_PUB_KEY),
          'cc_checkout_url' => esc_url(get_permalink($this->cc_checkout_page)),
          'id_checkout_url' => esc_url(get_permalink($this->id_checkout_page)),
          'redirect_url' => esc_url(get_permalink($this->check_payment_page)),
          'checkout_cart' => esc_url(get_permalink($this->checkout_page)),
          'connected' => json_encode($this->connected),
          'timeout' => 10000
        );
      }else{
        $this->globalObject = array(
          'homeUrl' => esc_url(home_url()),
          'connected' => false
        );
      }
    });
    add_action('admin_menu', function(){
      add_menu_page('Five Minute Webshop', 'Five Minute Webshop', 'manage_options', 'wp_sb', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/general_settings.php';
      }, 'dashicons-cart', 98);
      add_submenu_page('wp_sb', __('Manage Products', 'wp-stripe-bancontact'), __('Manage Products', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_products', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/settings_products.php';
      });
      add_submenu_page('wp_sb', __('Add Product', 'wp-stripe-bancontact'), __('Add Product', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_add_product', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/add_product.php';
      });
      add_submenu_page('fmwes_products', __('Edit Product', 'wp-stripe-bancontact'), __('Edit Product', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_edit_product', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/edit_product.php';
      });
      add_submenu_page('wp_sb', __('Manage Orders', 'wp-stripe-bancontact'), __('Manage Orders', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_orders', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/settings_orders.php';
      });
      add_submenu_page('wp_sb', __('Manage Coupons', 'wp-stripe-bancontact'), __('Manage Coupons', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_coupons', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/settings_coupons.php';
      });
      add_submenu_page('wp_sb', __('Add Coupon', 'wp-stripe-bancontact'), __('Add Coupon', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_add_coupon', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/add_coupon.php';
      });
      add_submenu_page('fmwes_coupons', __('Edit Coupon', 'wp-stripe-bancontact'), __('Edit Coupon', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_edit_coupon', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/edit_coupon.php';
      });
      add_submenu_page('wp_sb', __('Shipping', 'wp-stripe-bancontact'), __('Shipping', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_shipping', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/settings_shipping.php';
      });
      add_submenu_page('fmwes_orders', __('Order Overview', 'wp-stripe-bancontact'), __('Order Overview', 'wp-stripe-bancontact'), 'manage_options', 'fmwes_settings_order', function(){
          require FMWES_PLUGIN_DIR . 'includes/pages/settings_order.php';          
      });
    });
    add_action('admin_init', function(){
      register_setting('fmwes-settings', 'fmwes_cart_icon_menu');
      register_setting('fmwes-settings', 'fmwes_checkout_page');
      register_setting('fmwes-settings', 'fmwes_cc_checkout_page');
      register_setting('fmwes-settings', 'fmwes_id_checkout_page');
      register_setting('fmwes-settings', 'fmwes_check_payment_page');
      register_setting('fmwes-settings-hidden', 'fmwes_token');
      register_setting('fmwes-settings-hidden', 'fmwes_access_token');
      register_setting('fmwes-settings-hidden', 'fmwes_refresh_token');
      register_setting('fmwes-settings', 'fmwes_cart_color');
      register_setting('fmwes-settings', 'fmwes_bcc_email');
    });
    add_action('wp_enqueue_scripts', function(){
      global $post;
      if (
        is_a($post, 'WP_POST')
        && (has_shortcode($post->post_content, 'checkout_bc')
          || has_shortcode($post->post_content, 'checkout_cc')
          || has_shortcode($post->post_content, 'checkout_id'))
      ) {
        wp_enqueue_script('stripe', 'https://js.stripe.com/v3/', false);
        wp_register_script('bancontact', FMWES_PLUGIN_URL . 'js/bancontact.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('bancontact', 'globalObject', $this->globalObject);
        wp_localize_script('bancontact', 'localize', $this->get_localized_strings());
        wp_enqueue_script('bancontact');
        wp_register_script('cart', FMWES_PLUGIN_URL . 'js/cart.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('cart', 'globalObject', $this->globalObject);
        wp_localize_script('cart', 'localize', $this->get_localized_strings());
        wp_enqueue_script('cart');
        wp_register_style('cartstyle', FMWES_PLUGIN_URL. 'css/cart.css', array(), FMWES_PLUGIN_VERSION);
        wp_enqueue_style('cartstyle');
      }
      if(is_a($post, 'WP_POST') && has_shortcode($post->post_content, 'checkout_cc')){
        wp_register_script('card', FMWES_PLUGIN_URL . 'js/card.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('card', 'globalObject', $this->globalObject);
        wp_localize_script('card', 'localize', $this->get_localized_strings());
        wp_enqueue_script('card');       
      }
      if(is_a($post, 'WP_POST') && has_shortcode($post->post_content, 'checkout_id')){
        wp_register_script('ideal', FMWES_PLUGIN_URL . 'js/ideal.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('ideal', 'globalObject', $this->globalObject);
        wp_localize_script('ideal', 'localize', $this->get_localized_strings());
        wp_enqueue_script('ideal');
      }
      if( is_a($post, 'WP_POST') && has_shortcode($post->post_content, 'confirmation_bc')){
        wp_register_script('confirmation', FMWES_PLUGIN_URL . 'js/confirmation.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('confirmation', 'globalObject', $this->globalObject);
        wp_localize_script('confirmation', 'localize', $this->get_localized_strings());
        wp_enqueue_script('confirmation');
      }   
      if( is_a($post, 'WP_POST') && (has_shortcode($post->post_content, 'simple_product') || has_shortcode($post->post_content, 'fmwes_donation'))){
        wp_register_script('simple-product', FMWES_PLUGIN_URL . 'js/simple-product.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('simple-product', 'localize', $this->get_localized_strings());
        wp_enqueue_script('simple-product');
        wp_register_style('cartstyle', FMWES_PLUGIN_URL. 'css/cart.css', array(), FMWES_PLUGIN_VERSION);
        wp_enqueue_style('cartstyle');
        wp_register_script('cart', FMWES_PLUGIN_URL . 'js/cart.js', array('jquery'), FMWES_PLUGIN_VERSION);
        wp_localize_script('cart', 'globalObject', $this->globalObject);
        wp_localize_script('cart', 'localize', $this->get_localized_strings());
        wp_enqueue_script('cart');
      }
    });
    add_action('admin_enqueue_scripts', function () {
      $client_id = FMWES_APP_ID;
      $globalObject = array_merge($this->globalObject, array(
        'middleUrl' => FMWES_MIDDLE_URL_AJAX,
        'client_id' => $client_id,
        'token' => get_option('fmwes_token'),
        'ajaxurl' => admin_url('admin-ajax.php')
      ));
      if(isset($_GET["page"])){
        if($_GET["page"] == "fmwes_products" || $_GET["page"] == "fmwes_add_product" || $_GET["page"] == "fmwes_edit_product"){
          wp_register_script('product', FMWES_PLUGIN_URL . 'js/products.js', array('jquery'), FMWES_PLUGIN_VERSION);
          wp_localize_script('product', 'globalObject', $globalObject);
          wp_localize_script('product', 'localize', $this->get_localized_strings());
          wp_enqueue_script('product');
        }
        if($_GET["page"] == "fmwes_coupons" || $_GET["page"] == "fmwes_add_coupon" || $_GET["page"] == "fmwes_edit_coupon"){
          wp_register_script('coupon', FMWES_PLUGIN_URL . 'js/coupon.js', array('jquery'), FMWES_PLUGIN_VERSION);
          wp_localize_script('coupon', 'globalObject', $globalObject);
          wp_localize_script('coupon', 'localize', $this->get_localized_strings());
          wp_enqueue_script('coupon');
        }      
        if($_GET["page"] == "fmwes_orders" || $_GET["page"] == "fmwes_settings_order"){
          wp_register_script('order', FMWES_PLUGIN_URL . 'js/orders.js', array('jquery'), FMWES_PLUGIN_VERSION);
          wp_localize_script('order', 'globalObject', $globalObject);
          wp_localize_script('order', 'localize', $this->get_localized_strings());
          wp_enqueue_script('order');
        }
        if($_GET["page"] == "fmwes_shipping"){
          wp_register_script('shipping', FMWES_PLUGIN_URL . 'js/shipping.js', array('jquery'), FMWES_PLUGIN_VERSION);
          wp_localize_script('shipping', 'localize', $this->get_localized_strings());
          wp_localize_script('shipping', 'globalObject', $globalObject);
          wp_enqueue_script('shipping');
        }
      }
      wp_register_style('adminstyle', FMWES_PLUGIN_URL . 'css/admin.css', array(), FMWES_PLUGIN_VERSION);
      wp_enqueue_style('adminstyle');

      wp_enqueue_script('jquery-ui-datepicker');
      wp_register_style('jquery-ui', FMWES_PLUGIN_URL . 'css/jquery-ui.css', array(), FMWES_PLUGIN_VERSION);
      wp_enqueue_style('jquery-ui');

      wp_enqueue_media();
      wp_register_script('media-selector', FMWES_PLUGIN_URL . 'js/media-selector.js', array('jquery'), FMWES_PLUGIN_VERSION);
      wp_localize_script('media-selector', 'localize', $this->get_localized_strings());
      wp_enqueue_script('media-selector');

      wp_register_script('settings', FMWES_PLUGIN_URL . 'js/settings.js', array('jquery'), FMWES_PLUGIN_VERSION);
      wp_localize_script('settings', 'localize', $this->get_localized_strings());
      wp_localize_script('settings', 'globalObject', $globalObject);
      wp_enqueue_script('settings');
      
    });
    
    add_filter('plugin_action_links_'.FMWES_PLUGIN_NAME, function($links){
      $settings_link = '<a href="admin.php?page=wp_sb&tab=general-settings">'.esc_html__('Settings', 'wp-stripe-bancontact').'</a>';
      array_unshift($links, $settings_link);
      return $links;
    });
    add_action('rest_api_init', function(){
      if(class_exists('Products_Controller')){
          $productController = new Products_Controller(new Functions());
          $productController->register_routes();
      }
      if(class_exists('Hook_Controller')){
          $hookController = new Hook_Controller(new Functions());
          $hookController->register_routes();
      }
      if(class_exists('Orders_Controller')){
        $ordersController = new Orders_Controller(new Functions(), new HttpFunctions());
        $ordersController->register_routes();
      }
      if(class_exists('Coupons_Controller')){
        $couponsController = new Coupons_Controller(new Functions());
        $couponsController->register_routes();
      }
      if(class_exists('Connect_Controller')){
        $connectController = new Connect_Controller(new Functions(), new HttpFunctions());
        $connectController->register_routes();
      }
      if(class_exists('Shipping_Controller')){
          $shippingController = new Shipping_Controller(new Functions());
          $shippingController->register_routes();
      }
    });
    add_shortcode('simple_product', function($atts){
      $atts = shortcode_atts(
        array(
          'id' => 0,
          'quantity' => 0
        ), $atts, 'simple_product');
      $SimpleProduct = new SimpleProduct();
      return $SimpleProduct->show($atts['id'], $atts['quantity']);
    });
    add_shortcode('checkout_bc', function(){
      $checkout = new Checkout();
      return $checkout->show();
    });
    add_shortcode('confirmation_bc', function(){
      $confirmation = new Confirmation();
      return $confirmation->show();
    });
    add_shortcode('checkout_cc', function(){
      $cc = new CheckoutCreditCard();
      return $cc->show();
    });
    add_shortcode('checkout_id', function(){
      $id = new CheckoutiDeal();
      return $id->show();
    });

    add_shortcode('fmwes_donation', function(){    
      $Donation  = new Donation();
      return $Donation->show(); 
    });
    register_activation_hook(FMWES_FILENAME, function(){
      if(!post_exists(__('Checkout Cart', 'wp-stripe-bancontact'))){
        $my_post = array(
          'post_title' => wp_strip_all_tags(__('Checkout Cart', 'wp-stripe-bancontact')),
          'post_content' => '[checkout_bc]',
          'post_status' => 'publish',
          'post_author' => 1,
          'post_type' => 'page'
        );
        $result = wp_insert_post($my_post);
        if($result > 0 || !is_a($result, 'WP_Error') ){
          add_option('fmwes_checkout_page', $result);
        }
      } else{
        add_option('fmwes_checkout_page', get_page_by_title(__('Checkout Cart', 'wp-stripe-bancontact'))->ID);
      }
      if(!post_exists(__('Creditcard Checkout', 'wp-stripe-bancontact'))){
        $my_post = array(
          'post_title' => wp_strip_all_tags(__('Creditcard Checkout', 'wp-stripe-bancontact')),
          'post_content' => '[checkout_cc]',
          'post_status' => 'publish',
          'post_author' => 1,
          'post_type' => 'page'
        );
        $result = wp_insert_post($my_post);
        if($result > 0 || !is_a($result, 'WP_Error') ){
          add_option('fmwes_cc_checkout_page', $result);
        }
      }else{
        add_option('fmwes_cc_checkout_page', get_page_by_title(__('Creditcard Checkout', 'wp-stripe-bancontact'))->ID);
      }
      if (!post_exists(__('iDeal Checkout', 'wp-stripe-bancontact'))) {
        $my_post = array(
          'post_title' => wp_strip_all_tags(__('iDeal Checkout', 'wp-stripe-bancontact')),
          'post_content' => '[checkout_id]',
          'post_status' => 'publish',
          'post_author' => 1,
          'post_type' => 'page'
        );
        $result = wp_insert_post($my_post);
        if ($result > 0 || !is_a($result, 'WP_Error')) {
          add_option('fmwes_id_checkout_page', $result);
        }
      }else{
        add_option('fmwes_id_checkout_page', get_page_by_title(__('iDeal Checkout', 'wp-stripe-bancontact'))->ID);
      }
      if(!post_exists(__('Check Payment', 'wp-stripe-bancontact'))){
        $my_post = array(
          'post_title' => wp_strip_all_tags(__('Check Payment', 'wp-stripe-bancontact')),
          'post_content' => '[confirmation_bc]',
          'post_status' => 'publish',
          'post_author' => 1,
          'post_type' => 'page'
        );
        $result = wp_insert_post($my_post);
        if($result > 0 || !is_a($result, 'WP_Error') ){
          add_option('fmwes_check_payment_page', $result);
        }
      }else{
        add_option('fmwes_check_payment_page', get_page_by_title(__('Check Payment', 'wp-stripe-bancontact'))->ID);
      }
      add_option('fmwes_cart_color', '#000000');
      add_option('fmwes_cart_icon_menu', 'None');
    });
    add_filter('query_vars', function($qvars){
      $qvars[] .= 'pname';
      $qvars[] .= 'pemail';
      $qvars[] .= 'porder';
      return $qvars;
    });
    
    add_action('admin_notices', function (){
		if(!is_ssl()){
			?><div class="notice notice-error ">
				<p><?php echo esc_html__('The Five Minute Webshop plugin requires an SSL connection.', 'wp-stripe-bancontact')?></p>
			</div>
			<?php
		}
		if(!get_option('fmwes_refresh_token')){
		  ?><div class="notice notice-warning is-dismissible">
		  	<p><?php echo esc_html__('You need to connect to Stripe to use the Five Minute Webshop plugin. You can do this in the ', 'wp-stripe-bancontact');?><a href="<?php echo home_url() . "/wp-admin/admin.php?page=wp_sb";?>"><?php echo esc_html__('settings', 'wp-stripe-bancontact');?></a>
		  </div><?php 
    }else{
      if(!get_option('fmwes_payment_settings_notice', false)){
    ?><div class="notice notice-warning is-dismissible fmwes-dismissible" id="fmwes_payment_settings_notice">
      <p><?php echo esc_html__("Don't forget to allow Bancontact and iDeal in your Stripe account. You can do this via this link: ", "wp-stripe-bancontact")?> <a href="https://dashboard.stripe.com/settings/payments"><?php echo esc_html__("Stripe Payment Settings", 'wp-stripe-bancontact');?></a></p>
    </div>
    <?php
      }
    }
	});
    
  add_action('wp_ajax_dismiss_notice', function(){
    if(!isset($_POST['notice'])){
      die();
    }
    $notice = sanitize_text_field($_POST['notice']);
    update_option($notice, true);
    die();
  });

	
  }

  public function filter_nav_menus(){
    add_filter('wp_nav_menu_items', function($items, $args){
      if($args->menu->term_id == get_option('fmwes_cart_icon_menu')){
        $cc = get_option('fmwes_cart_color', '#000000');
        $items .= '<li class="menu_item"><a id="menu-cart" href="'.get_permalink(get_post(get_option('fmwes_checkout_page'))).'">'.
        '<svg x="0px" y="0px" viewBox="0 0 32 32" style="enable-background:new 0 0 32 32;"><path class="st0" fill="'.$cc.'"  d="M27.7,20.3l3.6-10.7c0.2-0.5,0-0.8-0.1-1c-0.4-0.5-1.1-0.5-1.3-0.5H9.5L9,5.3C8.9,4.7,8.4,4,7.5,4H1.8
        	C1.2,4,1,4.6,1,5.1v1.5C1,7.2,1.2,7,1.8,7h4.8l3.7,15.9c-0.6,0.6-0.9,1.6-0.9,2.5c0,1.9,1.5,3.5,3.4,3.5c1.8,0,3.1-1.8,3.4-2.8h7.2
        	c0.2,1,1.3,2.9,3.3,2.9c1.9,0,3.4-1.5,3.4-3.4S29,22,26.7,22c-0.9,0-2,0.1-2.5,1.1h-8.9c-0.6-1-1.5-1.3-2.4-1.3L12.8,21h13.5
        	C27.3,21,27.5,20.8,27.7,20.3z M11.5,25.5c0-0.7,0.6-1.3,1.3-1.3s1.3,0.6,1.3,1.3s-0.6,1.3-1.3,1.3S11.5,26.2,11.5,25.5z M26.8,26.8
        	c-0.7,0-1.3-0.6-1.3-1.3s0.6-1.3,1.3-1.3s1.3,0.6,1.3,1.3C28.1,26.2,27.5,26.8,26.8,26.8z"/></svg>'.
        '</a></li>';
      }
      return $items;
    }, 10, 2);
  }

  public function get_localized_strings(){
    $strings = array(
      'error' => __('Something went wrong. Please contact site administrator.', 'wp-stripe-bancontact'),
      'error_checking_payment' => __('Something went wrong checking your payment. Please check your email for order confirmation.', 'wp-stripe-bancontact'),
      'payment_failed' => __("We're sorry but your payment was not successful. Please try again.", 'wp-stripe-bancontact'),
      'payment_canceled' => __('Your order was canceled. ', 'wp-stripe-bancontact'),
      'payment_successful' => __('Payment Successful!', 'wp-stripe-bancontact'),
      'redirecting' => __('You are being redirected to the home page', 'wp-stripe-bancontact'),
      'rechecking_payment' => __('Rechecking Payment', 'wp-stripe-bancontact'),
      'delete' => __('Delete', 'wp-stripe-bancontact'),
      'confirm_delete_product' => __("Are you sure you want to delete this product?", 'wp-stripe-bancontact'),
      'code_mandatory' => __("You must set a code", 'wp-stripe-bancontact'),
      'fixed_amount_percentage_not_set' => __("you must set either a fixed amount or a percentage", 'wp-stripe-bancontact'),
      'fixed_amount_percentage_both_set' => __("you must set either a fixed amount or a percentage, not both", 'wp-stripe-bancontact'),
      'fixed_amount_not_numeric' => __("Fixed amount must be numeric", 'wp-stripe-bancontact'),
      'percentage_not_numeric' => __("Percentage must be numeric", 'wp-stripe-bancontact'),
      'quantity_not_set' => __("you must set a quantity", 'wp-stripe-bancontact'),
      'quantity_not_numeric' => __("Quantity must be numeric", 'wp-stripe-bancontact'),
      'coupon' => __("Coupon", 'wp-stripe-bancontact'),
      'coupon_used_up' => __("Coupon has been used up", 'wp-stripe-bancontact'),
      'coupon_expired' => __("Coupon has expired", 'wp-stripe-bancontact'),
      'coupon_code_emtpy' => __('Please provide a coupon code', 'wp-stripe-bancontact'),
      'coupon_not_found' => __('Coupon code invalid', 'wp-stripe-bancontact'),
      'confirm_delete_coupon' => __('Are you sure you want to delete this coupon?', 'wp-stripe-bancontact'),
      'select_image' => __("Select image to upload", 'wp-stripe-bancontact'),
      'use_image' => __("Use this image", 'wp-stripe-bancontact'),
      'copied' => __("Copied!", 'wp-stripe-bancontact'),
      'connected' => __("Account Connected", 'wp-stripe-bancontact'),
      'name' => __('Name', 'wp-stripe-bancontact'),
      'unit_of_measure' => __('Unit Of Measure', 'wp-stripe-bancontact'),
      'value' => __('Value', 'wp-stripe-bancontact'),
      'less_info' =>__('Less Info', 'wp-stripe-bancontact'),
      'more_info' => __('More Info', 'wp-stripe-bancontact'),
      'shipping_cost' => __('Shipping Cost', 'wp-stripe-bancontact'),
      'cost' => __('Cost', 'wp-stripe-bancontact'),
      'saved' => __('Saved', 'wp-stripe-bancontact'),
      'shipping' => __('Shipping', 'wp-stripe-bancontact'),
      'free_shipping' => __('Free Shipping', 'wp-stripe-bancontact'),
      'no_shipping' => __('We\'re sorry, we do not offer shipping to your country.', 'wp-stripe-bancontact'),   
      'donation' => __('donation', 'wp-stripe-bancontact')
    );
    return $strings;
  }
  
  function generatedStrings(){
    array(
        'handled' => __('handled', 'wp-stripe-bancontact'),
        'payment_succeeded' =>  __('payment_succeeded', 'wp-stripe-bancontact'),
        'created' => __('created', 'wp-stripe-bancontact'),
        'payment_canceled' => __('payment_canceled', 'wp-stripe-bancontact'),
        'payment_failed' => __('payment_failed', 'wp-stripe-bancontact')
    );
  }
  
  

}
