<?php
/**
 * custom class for displaying orders
 *
 * @package WP_Stripe_Bancontact
 */

class FMWES_List_Table_Orders extends WP_List_Table_C{
    
    private $http_functions;
    private $account_id;
    
  public function __construct( $args = array(), $_http_functions) {
		parent::__construct( $args );
		$this->http_functions = $_http_functions;
	}
	
	protected function get_views() {
	    $views = array();
	    $current = ( !empty($_REQUEST['status'])?$_REQUEST['status'] : 'all');
	    
	    $class=($current == 'all'? ' class="current"': '');
	    $all_url = remove_query_arg('status');
	    $views['all'] = "<a href='{$all_url}' {$class}>".esc_html__('All', 'wp-stripe-bancontact')."</a>";
	    
	    $class=($current == 'payment_succeeded'? ' class="current"': '');
	    $ps_url = add_query_arg('status', 'payment_succeeded');
	    $views['payment_succeeded'] = "<a href='{$ps_url}' {$class}>".esc_html__('Payment Succeeded', 'wp-stripe-bancontact')."</a>";
	    
	    $class=($current == 'created'? ' class="current"': '');
	    $created_url = add_query_arg('status', 'created');
	    $views['created'] = "<a href='{$created_url}' {$class}>".esc_html__('Awaiting Payment', 'wp-stripe-bancontact')."</a>";
	    
	    $class=($current == 'handled'? ' class="current"': '');
	    $handled_url = add_query_arg('status', 'handled');
	    $views['handled'] = "<a href='{$handled_url}' {$class}>".esc_html__('Handled', 'wp-stripe-bancontact')."</a>";
	    
	    $class=($current == 'payment_canceled'? ' class="current"': '');
	    $canceled_url = add_query_arg('status', 'payment_canceled');
	    $views['payment_canceled'] = "<a href='{$canceled_url}' {$class}>".esc_html__('Payment Canceled', 'wp-stripe-bancontact')."</a>";
	    
	    $class=($current == 'payment_failed'? ' class="current"': '');
	    $failed_url = add_query_arg('status', 'payment_failed');
	    $views['payment_failed'] = "<a href='{$failed_url}' {$class}>".esc_html__('Payment Failed', 'wp-stripe-bancontact')."</a>";
	    return $views;
	}

	function get_columns() {
		$columns = array(
			'name' => esc_html__('Name', 'wp-stripe-bancontact'),
      'email' => esc_html__('Email', 'wp-stripe-bancontact'),
			'total_price' => esc_html__('Amount', 'wp-stripe-bancontact'),
			'timestamp' => esc_html__('Date', 'wp-stripe-bancontact'),
      'type' => esc_html__('Type', 'wp-stripe-bancontact'),
      'order_status' => esc_html__('Status', 'wp-stripe-bancontact')
		);
		return $columns;
	}

  function prepare_items(){
    if(!empty($_GET['action'])){
      $id = sanitize_text_field($_GET['id']);
      if(is_numeric($id)){
        if($_GET['action'] == 'delete' && !empty($id)){
          $this->delete_item($id);
        }
      }
    }
    $this->account_id = $this->http_functions->getAccountId();
    $columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);
		$orderby = (! empty( $_GET['orderby'])) ? sanitize_text_field($_GET['orderby']): 'name';
		$order = (!empty($_GET['order']))? sanitize_text_field($_GET['order']):'asc';
		$search = (!empty($_POST['s']))? sanitize_text_field($_POST['s']):'';
		$current_page = $this->get_pagenum();
		$total_items = $this->get_total_items($search);
		$this->set_pagination_args(array(
			'total_items' => $total_items,
			'per_page' => 10
		));
	$test_mode = 0;
	$statusfilter = (isset($_REQUEST['status'])?$_REQUEST['status'] : 'all');
	$this->items = $this->fetch_orders($orderby, $order, $current_page, $search, $test_mode, $statusfilter);
  }

	function column_default( $item, $column_name ) {
	    if($column_name == 'order_status'){
	        return esc_html__($item->$column_name, 'wp-stripe-bancontact');
	    }else{
		  return $item->$column_name;
	    }
	}

  function fetch_orders($orderby, $order, $page, $search, $test_mode, $statusfilter){
    global $wpdb;
    $andstatus = '';
    if($statusfilter != 'all'){
        $andstatus = ' and order_status = "' . $statusfilter .'"';
    }
    $where = $search ==''?'where (test='.$test_mode.' or test IS NULL)':"where name like '%".$search."%' and (test=".$test_mode." or test IS NULL)";
    $where .= $andstatus;
    return $wpdb->get_results('select * from fmwes_orders ' . $where.
                              ' order by '
                              .$orderby. ' ' .$order
                              . ' LIMIT ' . (($page - 1) * 10).',10');
  }

  function column_name($item){
    $a_test = FMWES_TEST_MODE==true?'test/':'';
    $a_href_stripe = sprintf("https://dashboard.stripe.com/".$this->account_id.'/'.$a_test."payments/%s", urlencode($item->payment_intent_id));
    $actions = array(
        'stripe' => '<a href="'.esc_html($a_href_stripe).'">'.esc_html__('Stripe payment', 'wp-stripe-bancontact').'</a>',
        'overview' => sprintf('<a href="'.esc_html(home_url()).'/wp-admin/admin.php?page=fmwes_settings_order&id=%s'.'">'.esc_html__('Overview','wp-stripe-bancontact').'</a>', urlencode($item->id))
		);
		return sprintf('%1$s %2$s', esc_html($item->name), $this->row_actions($actions));
  }

  function get_total_items($search){
    global $wpdb;
		$where = $search==''?'':"where name like '%".$search."%'";
		return $wpdb->get_var('select count(*) from fmwes_orders '.$where);
  }

  function get_sortable_columns() {
		$sortable_columns = array(
			'name' => array('name', false),
      'email' => array('email', false),
			'total_price' => array('total_price', false),
			'timestamp' => array('timestamp', false),
      'type' => array('type', false),
      'order_status' => array('order_status', false)
		);
		return $sortable_columns;
	}
}
