<?php
/**
 * custom class for displaying products
 *
 * @package WP_Stripe_Bancontact
 */

class FMWES_List_Table_Products extends WP_List_Table_C{


	public function __construct( $args = array()) {
		parent::__construct( $args );
	}

	function get_columns() {
		$columns = array(
			'title' => esc_html__('Title', 'wp-stripe-bancontact'),
			'unit_price' => esc_html__('Unit Price', 'wp-stripe-bancontact'),
			'VAT_percentage' => esc_html__('VAT Percentage', 'wp-stripe-bancontact'),
			'shortcode' => esc_html__('Shortcode', 'wp-stripe-bancontact')
		);
		return $columns;
	}

	function prepare_items() {
		if(!empty($_GET['action'])){
			$id = sanitize_text_field($_GET['id']);
			if(is_numeric($id)){
				if($_GET['action'] == 'delete' && !empty($id)){
					$this->delete_item($id);
				}
			}
		}
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();
		$this->_column_headers = array($columns, $hidden, $sortable);
		$orderby = (! empty( $_GET['orderby'])) ? sanitize_text_field($_GET['orderby']): 'title';
		$order = (!empty($_GET['order']))? sanitize_text_field($_GET['order']):'asc';
		$search = (!empty($_POST['s']))? sanitize_text_field($_POST['s']):'';
		$current_page = $this->get_pagenum();
		$total_items = $this->get_total_items($search);
		$this->set_pagination_args(array(
			'total_items' => $total_items,
			'per_page' => 10
		));
		$this->items = $this->fetch_products($orderby, $order, $current_page, $search);
	}

	function column_default( $item, $column_name ) {
		return $item->$column_name;
	}


	function fetch_products($orderby, $order, $page, $search){
		global $wpdb;
		$where = $search==''?'':"where title like '%".$search."%'";
		return $wpdb->get_results('select * from fmwes_products '.$where.
		                          ' order by '
		                          . $orderby . ' ' . $order
		                          . ' LIMIT '.(($page - 1) * 10).',10');
	}

	function column_title($item){
		$actions = array(
			'delete' => sprintf('<a href="" onclick="deleteProduct(%s, event);">'.esc_html__('Delete', 'wp-stripe-bancontact').'</a>', urlencode($item->id)),
		    'edit' => sprintf('<a href="'.esc_html(home_url()).'/wp-admin/admin.php?page=fmwes_edit_product&id=%s'.'">'.esc_html__('Edit', 'wp-stripe-bancontact').'</a>', urlencode($item->id)),
		);
		return sprintf('%1$s %2$s', esc_html($item->title), $this->row_actions($actions));
	}

	function column_shortcode($item){
		return sprintf('<div id="copy-div-%1$s">[simple_product id=%1$s]</div><a href="" onclick="copyToClipboard(%1$s)">'.esc_html__('Copy to clipboard', 'wp-stripe-bancontact').'</a>', $item->id );
	}

	function get_total_items($search){
		global $wpdb;
		$where = $search==''?'':"where title like '%".$search."%'";
		return $wpdb->get_var('select count(*) from fmwes_products '.$where);
	}

	function get_sortable_columns() {
		$sortable_columns = array(
			'title' => array('title', false),
			'unit_price' => array('unit_price', false),
			'VAT_percentage' => array('VAT_percentage', false)
		);
		return $sortable_columns;
	}


}
