<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Connect_Controller extends WP_REST_Controller {
  private $functions;
  private $http_functions;

  public function __construct($_functions, $_http_functions){
    $this->functions = $_functions;
    $this->http_functions = $_http_functions;
  }
  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'connect';
    register_rest_route($namespace, '/' . $base , array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'connect'),
        'args' => array()
      )
    ));
    register_rest_route($namespace, '/disconnect', array(
      array(
        'methods' => WP_REST_Server::DELETABLE,
        'callback' => array($this, 'disconnect'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
  }

  public function connect($request){
    try{
      global $wpdb;
      $parameters = (array) json_decode( $request->get_body() );
      $token = sanitize_text_field($parameters['token']);
      if($token != get_option('fmwes_token')){
        return new WP_REST_Response(array('message' => 'Unauthorized'), 401);
      }
      $access_token = sanitize_text_field($parameters['access_token']);
      $refresh_token = sanitize_text_field($parameters['refresh_token']);
      if(!$access_token){
        throw new InvalidArgumentException('Access_token is empty');
      }
      if(!$refresh_token){
        throw new InvalidArgumentException('refresh_token is empty');
      }
      update_option('fmwes_access_token', $access_token);
      update_option('fmwes_refresh_token', $refresh_token);
      return new WP_REST_Response('ok', 200);
    } catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		} catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function disconnect($request){
    try{
      update_option('fmwes_refresh_token', '');
      $url = FMWES_MIDDLE_URL . '/disconnect';
      $result = $this->http_functions->makeHttpRequest('DELETE', $url, null, 'live');
      if($result['response']['code'] != 200){
        throw new Exception('Unable to disconnect');
      } 
      return new WP_REST_Response(array('message' => 'ok', 'status' => 200), 200);
    } catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		} catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }
}
