<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Coupons_Controller extends WP_REST_Controller {
  private $table = 'fmwes_coupons';
  private $functions;

  public function __construct($_functions){
    $this->functions = $_functions;
  }

  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'coupon';
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'create_coupon'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::EDITABLE,
        'callback' => array($this, 'edit_coupon'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/(?P<id>[\d]+)', array(
      array(
        'methods' => WP_REST_Server::DELETABLE,
        'callback' => array($this, 'delete_coupon'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/validate', array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'callback' => array($this, 'validate'),
        'args' => array(
          'code' => array(
            'default' => ''
          )
        ),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));


  }

  public function create_coupon($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array)json_decode($request->get_body());
      $this->validate_coupon($parameters);
      $code = sanitize_text_field($parameters['code']);
      $fixed_amount = $parameters['fixed_amount'];
      $percentage = $parameters['percentage'];
      $total_quantity = $parameters['total_quantity'];
      $e_date = DateTime::createFromFormat('d-m-Y', $parameters['expiration_date']);
      $fa_not_set = $fixed_amount == null || $fixed_amount == "";
      $p_not_set = $percentage == null || $percentage == "";
      $rowcount = $wpdb->get_var("Select count(*) from ". $this->table." where code = '".$code."'");
      if($rowcount > 0){
        throw new InvalidArgumentException('A coupon already exists with the same code');
      }
      $quantity_remaining = $total_quantity;
      $data = array(
        'code' => $code,
        'fixed_amount' => $fixed_amount,
        'percentage' => $percentage,
        'total_quantity' => $total_quantity,
        'quantity_remaining' => $quantity_remaining,
        'expiration_date' => date_format($e_date, 'Y-m-d')
      );
      $res = $wpdb->insert($this->table, $data);
      if($res == 1 ){
        $wpdb->query('COMMIT');
        return new WP_REST_Response($res, 200);
      }
      $wpdb->query('ROLLBACK');
      throw new Exception('Can\'t create');
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function edit_coupon($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array) json_decode( $request->get_body() );
      $this->validate_coupon($parameters);
      $total_quantity = $parameters['total_quantity'];
      if(!is_numeric($parameters['id'])){
        throw new InvalidArgumentException('id wrong format');
      }
      $id = (int)$parameters['id'];
      $data = array(
        'total_quantity' => $total_quantity
      );
      $where = array('id' => $id);
      $res = $wpdb->update($this->table, $data, $where);
      if($res == 1){
        $wpdb->query('COMMIT');
        return new WP_REST_Response($data, 200);
      }
      $tmp = $wpdb->last_query;
      $wpdb->query('ROLLBACK');
      throw new Exception('can\'t edit');
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function delete_coupon($request){
    try{
      global $wpdb;
			if(!is_numeric($request['id'])){
				throw new InvalidArgumentException('id wrong format');
			}
			$id  = $request['id'];
			$res = $wpdb->delete( $this->table, array( 'id' => $id ) );
			if ( $res == 1 ) {
				return new WP_REST_Response( $id, 200 );
			}
			throw new Exception('can\'t delete');
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function validate($request){
    try{
      $code = sanitize_text_field($request['code']);
      $coupon = $this->functions->checkCoupon($code);
      return new WP_REST_Response($coupon, 200);
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  private function validate_coupon($parameters){
    $code = sanitize_text_field($parameters['code']);
    $fixed_amount = $parameters['fixed_amount'];
    $percentage = $parameters['percentage'];
    $total_quantity = $parameters['total_quantity'];
    $fa_not_set = $fixed_amount == null || $fixed_amount == "";
    $p_not_set = $percentage == null || $percentage == "";
    if($fa_not_set && $p_not_set){
      throw new InvalidArgumentException('Either fixed amount or percentage must be set');
    }
    if(!$fa_not_set && !$p_not_set){
      throw new InvalidArgumentException('Either fixed amount or percentage must be set, not both');
    }
    if(!$fa_not_set){
      if(!is_numeric($fixed_amount)){
        throw new InvalidArgumentException('Fixed amount must be numeric');
      }
    }
    else{
      $fixed_amount = 0;
    }
    if(!$p_not_set){
      if(!is_numeric($percentage)){
        throw new InvalidArgumentException('Percentage must be numeric');
      }
    }
    else{
      $percentage = 0;
    }
    if(!is_numeric($total_quantity)){
      throw new InvalidArgumentException('Total quantity must be numeric');
    }
    if(!$parameters['expiration_date']){
      throw new InvalidArgumentException('Expiration date is required');
    }
  }

}
