<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Orders_Controller extends WP_REST_Controller {
  private $table = 'fmwes_orders';
  private $functions;
  private $http_functions;

  public function __construct($_functions, $_http_functions){
    $this->functions = $_functions;
    $this->http_functions = $_http_functions;
    \Stripe\Stripe::setApiKey(esc_attr(get_option('fmwes_secret_key')));
    
  }

  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'order';
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'create_order'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base , array(
      array(
        'methods' => WP_REST_Server::EDITABLE,
        'callback' => array($this, 'edit_order'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base , array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'callback' => array($this, 'get_order'),
        'args' => array(
          'sourceid' => array(
            'default' => ''
          )
        ),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/intent', array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'callback' => array($this, 'create_intent'),
        'args' => array(
          'sourceid' => array(
            'default' => ''
          )
        ),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/source', array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'callback' => array($this, 'create_source'),
        'args' => array(
          'sourceid' => array(
            'default' => ''
          )
        ),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/product/status', array(
       array(
         'methods' => WP_REST_Server::EDITABLE,
         'callback' => array($this, 'update_product_status'),
         'args' => array(),
         'permission_callback' => array($this->functions, 'check_permission')
       ) 
    ));
    register_rest_route($namespace, '/' . $base . '/status', array(
       array(
           'methods' => WP_REST_Server::EDITABLE,
           'callback' => array($this, 'update_status'),
           'args' => array(),
           'permission_callback' => array($this->functions, 'check_permission')
       ) 
    ));
  }

  
  public function update_status($request){
      try{
          global $wpdb;
          $parameters = (array) json_decode( $request->get_body() );
          $id = -1;
          $data = array();
          if(array_key_exists('id', $parameters)){
              $id = sanitize_text_field($parameters['id']);
              if(!$id > 0){
                  throw new InvalidArgumentException('orderid not valid');
              }
          }else{
              throw new InvalidArgumentException('orderid missing');
          }
          if(array_key_exists('order_status', $parameters)){
              $status = sanitize_text_field($parameters['order_status']);
              if($status == 'payment_succeeded' || $status == 'handled'){
                  $data['order_status'] = $status;
              }else{
                  throw new InvalidArgumentException('status not valid');
              }
          }else{
              throw new InvalidArgumentException('status missing');
          }
          $where = array('id' => $id);
          $res = $wpdb->update($this->table, $data, $where);
          if($res != 1){
              throw new Exception('status update failed');
          }
          return new WP_REST_Response($data, 200);         
      }catch (InvalidArgumentException $iae){
          $this->functions->writeToLog($iae);
          return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
      }
      catch (Exception $e){
          $this->functions->writeToLog($e);
          return new WP_REST_Response(array('message' => $e->getMessage()), 500);
      }
  }
  
  public function update_product_status($request){
      try{
          global $wpdb;
          $parameters = (array) json_decode( $request->get_body() );
          $data = array();
          $orderid = -1;
          $productid = -1;
          if(array_key_exists('order_id', $parameters)){
              $orderid = sanitize_text_field($parameters['order_id']);
              if(!$orderid > 0){
                  throw new InvalidArgumentException('orderid not valid');
              }
          }else{
              throw new InvalidArgumentException('orderid missing');
          }
          if(array_key_exists('product_id', $parameters)){
              $productid = sanitize_text_field($parameters['product_id']);
              if(!$productid > 0){
                  throw new InvalidArgumentException('productid not valid');
              }
          }else{
              throw new InvalidArgumentException('productid missing');
          }
          if(array_key_exists('status', $parameters)){
              $status = sanitize_text_field($parameters['status']);
              if($status == '' || $status == 'collected'){
                  $data['status'] = $status;
              }else{
                  throw new InvalidArgumentException('status not valid');
              }
          }else{
              throw new InvalidArgumentException('status missing');
          }
          $where = array('order_id' => $orderid, 'product_id' => $productid);
          $res = $wpdb->update('fmwes_order_products', $data, $where);
          if($res != 1){
              throw new Exception('status update failed');
          }
          return new WP_REST_Response($data, 200);
      }catch (InvalidArgumentException $iae){
          $this->functions->writeToLog($iae);
          return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
      }
      catch (Exception $e){
          $this->functions->writeToLog($e);
          return new WP_REST_Response(array('message' => $e->getMessage()), 500);
      }
  }
  
  public function get_order($request){
    try{
      global $wpdb;
      $sourceid = sanitize_text_field($request['sourceid']);
      if(!$sourceid){
          $pi = sanitize_text_field($request['paymentintentid']);
          $res = $wpdb->get_row( 'select * from ' . $this->table . ' where payment_intent_id = "' . $pi .'"');
          if ( isset( $res ) ) {
              return new WP_REST_Response( $res, 200 );
          }
          throw new Exception('can\'t get');
      }
      else{
          $res = $wpdb->get_row( 'select * from ' . $this->table . ' where sourceid = "' . $sourceid .'"');
        	if ( isset( $res ) ) {
        		return new WP_REST_Response( $res, 200 );
        	}
        	throw new Exception('can\'t get');
      }
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function edit_order($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array) json_decode( $request->get_body() );
      if(!is_numeric($parameters['id'])){
        throw new InvalidArgumentException('id wrong format');
      }
      $id = (int)$parameters['id'];
      $name = sanitize_text_field($parameters['name']);
      $email = sanitize_text_field($parameters['email']);
      $status = sanitize_text_field($parameters['order_status']);
      $data = array(
        'name' => $name,
        'email' => $email
      );
      if(array_key_exists('sourceid', $parameters)){
        $sourceid = sanitize_text_field($parameters['sourceid']);
        $data['sourceid'] = $sourceid;
      }
      $where = array('id' => $id);
      $res = $wpdb->update($this->table, $data, $where);
      if($res == 1){
        $wpdb->query('COMMIT');
        return new WP_REST_Response($data, 200);
      }
      $wpdb->query('ROLLBACK');
      throw new Exception('can\'t edit');
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function create_order($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array) json_decode( $request->get_body() );
      $name = sanitize_text_field($parameters['name']);
      $email = sanitize_text_field($parameters['email']);
      $address_line_1 = sanitize_text_field($parameters['address_line_1']);
      $address_line_2 = sanitize_text_field($parameters['address_line_2']);
      $zip = sanitize_text_field($parameters['zip']);
      $city = sanitize_text_field($parameters['city']);
      $country = sanitize_text_field($parameters['country']);
      $type = sanitize_text_field($parameters['type']);
      $coupon = $parameters["coupon"];
      $discount = 0;
      $total_price = 0.0;
      $coupon_code = "";
      foreach($parameters['products'] as $product){
        if($product->donation === true){
          $total_price += $product->amount;
        }
        else{
          $dbproduct = $wpdb->get_row("SELECT * FROM fmwes_products where id=".sanitize_text_field($product->id));
          if($dbproduct == null){
            throw new Exception('product was not found');
          }
          if($dbproduct->VAT_percentage > 0){
            $total_price += ($dbproduct->unit_price * sanitize_text_field($product->quantity)) * (1 + ($dbproduct->VAT_percentage / 100));
          }
          else{
            $total_price += ($dbproduct->unit_price * sanitize_text_field($product->quantity));
          }
        }
      }
      $ship_all = $wpdb->get_row('select * from fmwes_shipping where country="*"');
      $shipping = "";
      if(!$ship_all){
          throw new Exception('Could not get shipping info');
      }else{
          if($ship_all->checked){
              $shipping = $ship_all;
          }else{
             $ship_country = $wpdb->get_row('select * from fmwes_shipping where country="'.$country.'"');
             if(!$ship_country){
                 throw new Exception('Could not get shipping info');
             }else{
                 if(!$ship_country->checked){
                     throw new Exception('No shipping to country');
                 }else{
                     $shipping = $ship_country;
                 }
             }
          }
      }
      $total_price += $shipping->cost;
      if($coupon != null){
        $coupon_code = sanitize_text_field($coupon->code);
        $verified_coupon = $this->functions->checkCoupon($coupon_code);
        $discount = $verified_coupon->fixed_amount!=0?$verified_coupon->fixed_amount:$total_price*($verified_coupon->percentage/100);
      }
      $total_price -= $discount;
      if($total_price < 0){
        $total_price = 0;
      }
      $data = array(
        'name' => $name,
        'email' => $email,
        'address_line_1' => $address_line_1,
        'address_line_2' => $address_line_2,
        'zip' => $zip,
        'city' => $city,
        'country' => $country,
        'total_price' => $total_price,
        'order_status' => 'created',
        'timestamp' => date('Y-m-d H:i:s'),
        'type' => $type,
        'coupon_code' => $coupon_code,
        'coupon_amount' => $discount
      );
      $res = $wpdb->insert($this->table, $data);
      $orderid = $wpdb->insert_id;
      $data['id'] = $orderid;
      if($res != 1 || $orderid <= 0){
        $wpdb->query('ROLLBACK');
        throw new Exception('Something went wrong inserting order');
      }
      foreach ($parameters['products'] as $product) {
        if($product->donation !== true){
          $ndata = array(
            'product_id' => sanitize_text_field($product->id),
            'order_id' => $orderid,
            'quantity' => sanitize_text_field($product->quantity)
          );
          $nres = $wpdb->insert('fmwes_order_products', $ndata);
          if(!$nres == 1){
            $wpdb->query('ROLLBACK');
            throw new Exception('Something went wrong inserting suborder of product '.$product->id);
          }
        }
      }
      $wpdb->query('COMMIT');
      $returnarray = array(
        'order' => $data
      );
      return new WP_REST_Response($returnarray, 200);
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function create_source($request){
    try{
      global $wpdb;
      $orderid = sanitize_text_field($request['orderid']);
      $order = $wpdb->get_row( 'select * from ' . $this->table . ' where id = "' . $orderid .'"');
      if ( !isset( $order ) ) {
        throw new Exception('can\'t get order');
      }
      $body = array(
          "order" => $order,
          "return_url" => esc_url(get_permalink(get_post(get_option('fmwes_check_payment_page'))))
      );
      $url = FMWES_MIDDLE_URL . '/source';
      $result = $this->http_functions->makeHttpRequest('POST', $url, $body);
      if($result['response']['code'] != 200){
          throw new Exception("Could not create source");
      }
      $source = json_decode($result['body']);
      if(!$source){
          throw new Exception('Source null');
      }  
      $res = $wpdb->update('fmwes_orders', array('sourceid' => $source->id), array('id' => $order->id));
      if($res == 1){
        return new WP_REST_Response($source, 200);
      }else{
        throw new Exception('Update failed');
      }
    }
    catch (InvalidArgumentException $iae){
      $this->functions->writeToLog($iae);
      return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
    }
    catch (Exception $e){
      $this->functions->writeToLog($e);
      return new WP_REST_Response(array('message' => $e->getMessage()), 500);
    }
  }

  public function create_intent($request){
    try{
      global $wpdb;
      $orderid = sanitize_text_field($request['orderid']);
      $order = $wpdb->get_row( 'select * from ' . $this->table . ' where id = "' . $orderid .'"');
			if ( !isset( $order ) ) {
        throw new Exception('can\'t get order');
      }
      $url = FMWES_MIDDLE_URL . '/intent';
      $body = array(
        'account_id' => get_option('fmwes_account_id'),
        'token' => get_option('fmwes_token'),
        'order' => $order
      );
      $result = $this->http_functions->makeHttpRequest('POST', $url, $body);
      if($result['response']['code'] != 200){
        throw new Exception("Could not create intent");    
      }
      $intent = json_decode($result['body']);
      if(!$intent){
        throw new Exception('Intent null');
      }
      $res = $wpdb->update('fmwes_orders', array('payment_intent_id' => $intent->id), array('id' => $order->id));
      $account_id = $this->http_functions->getAccountId();
      if($res == 1){
        return new WP_REST_RESPONSE(array('client_secret' => $intent->client_secret, 'account_id' => $account_id), 200);
      }else{
        throw new Exception('update failed');
      }

    }
    catch (InvalidArgumentException $iae){
      $this->functions->writeToLog($iae);
      return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
    }
    catch (Exception $e){
      $this->functions->writeToLog($e);
      return new WP_REST_Response(array('message' => $e->getMessage()), 500);
    }
  }

}
