<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Products_Controller extends WP_REST_Controller {
  private $table = 'fmwes_products';
  private $functions;

  public function __construct($_functions){
    $this->functions = $_functions;
  }

  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'product';
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'create_product'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::EDITABLE,
        'callback' => array($this, 'edit_product'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base . '/(?P<id>[\d]+)', array(
      array(
        'methods' => WP_REST_Server::DELETABLE,
        'callback' => array($this, 'delete_product'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base , array(
      array(
        'methods' => WP_REST_Server::READABLE,
        'callback' => array($this, 'get_product'),
        'args' => array(
          'id' => array(
            'default' => ''
          )
        ),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
  }

  public function get_product($request){
    try{
      global $wpdb;
      $id = sanitize_text_field($request['id']);
      if($id){
          $res = $wpdb->get_row( 'select * from ' . $this->table . ' where id = ' . $id);
          if ( isset( $res ) ) {
              $res->img_src = wp_get_attachment_image_src($res->image_attachment_id);
              return new WP_REST_Response( $res, 200 );
          }
          throw new Exception('can\'t get');
      }     
      else{
        throw new InvalidArgumentException('id wrong format');
      }    
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function create_product($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array)json_decode($request->get_body());
      $description = sanitize_text_field($parameters['description']);
      $unit_price = sanitize_text_field($parameters['unit_price']);
      $vat_percentage = sanitize_text_field($parameters['vat_percentage']);
      $title = sanitize_text_field($parameters['title']);
      $specs = $parameters['specs'];
      $image_attachment_id = sanitize_text_field($parameters['image_attachment_id']);
      $data = array(
        'description' => $description,
        'unit_price' => $unit_price,
        'VAT_percentage' => $vat_percentage,
        'title' => $title,
        'image_attachment_id' => $image_attachment_id
      );   
      $res = $wpdb->insert($this->table, $data);
      if($res != 1 ){
          $wpdb->query('ROLLBACK');
          throw new Exception('Can\'t create');
      }
      $product_id = $wpdb->insert_id;
      foreach ($specs as $spec){
          $spec_data = array(
              'name' => sanitize_text_field($spec->name),
              'spec_value' => sanitize_text_field($spec->spec_value),
              'unit_of_measure' => sanitize_text_field($spec->unit_of_measure),
              'product_id' => $product_id
          );
          $spec_res = $wpdb->insert('fmwes_product_specs', $spec_data);
          if($spec_res != 1){
              $wpdb->query('ROLLBACK');
              throw new Exception('Can\'t create spec');
          }
      }
      $wpdb->query('COMMIT');
      return new WP_REST_Response($data, 200);
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
	}
	catch (Exception $e){
		$this->functions->writeToLog($e);
		return new WP_REST_Response(array('message' => $e->getMessage()), 500);
	}
  }

  public function edit_product($request){
    try{
      global $wpdb;
      $wpdb->query('START TRANSACTION');
      $parameters = (array) json_decode( $request->get_body() );
      $description = sanitize_text_field($parameters['description']);
      $unit_price = sanitize_text_field($parameters['unit_price']);
      $vat_percentage = sanitize_text_field($parameters['vat_percentage']);
      $title = sanitize_text_field($parameters['title']);
      $image_attachment_id = sanitize_text_field($parameters['image_attachment_id']);
      $specs = $parameters['specs'];
      if(!is_numeric($parameters['id'])){
        throw new InvalidArgumentException('id wrong format');
      }
      $id = (int)$parameters['id'];
      $data = array(
        'description' => $description,
        'unit_price' => $unit_price,
        'VAT_percentage' => $vat_percentage,
        'title' => $title,
        'image_attachment_id' => $image_attachment_id
      );
      $where = array('id' => $id);
      $res = $wpdb->update($this->table, $data, $where);
      if($res === false){
          $wpdb->query('ROLLBACK');
          throw new Exception('Couldn\'t edit');
      }   
      $ospecs = $wpdb->get_results('select * from fmwes_product_specs where product_id='. $id);
      $to_insert = array();
      $to_update = array();
      $to_delete = array();
      foreach($ospecs as $ospec){
          $found = false;
          foreach($specs as $spec){
              if($spec->id == $ospec->id){
                  $to_update[] = $spec;
                  $found = true;
              }
          }
          if(!$found){
              $to_delete[] = $ospec;
          }
      }
      foreach ($specs as $spec) {
          if($spec->id <= 0){
              $to_insert[] = $spec;
          }
      }
      foreach($to_insert as $spec){
          $sdata = array(
              'name' => sanitize_text_field($spec->name),
              'spec_value' => sanitize_text_field($spec->spec_value),
              'unit_of_measure' => sanitize_text_field($spec->unit_of_measure),
              'product_id' => $id
          );
          $sres = $wpdb->insert('fmwes_product_specs', $sdata);
          if($sres != 1){
              $wpdb->query('ROLLBACK');
              throw new Exception('Can\'t create spec');
          }
      }
      foreach($to_update as $spec){
          $sdata = array(
              'name' => sanitize_text_field($spec->name),
              'spec_value' => sanitize_text_field($spec->spec_value),
              'unit_of_measure' => sanitize_text_field($spec->unit_of_measure)
          );
          $sres = $wpdb->update('fmwes_product_specs', $sdata, array('id' => $spec->id));
          if($sres != 1){
              $wpdb->query('ROLLBACK');
              throw new Exception('Can\'t update spec');
          }
      }
      foreach($to_delete as $spec){
          $sres = $wpdb->delete ('fmwes_product_specs', array('id' => $spec->id));
          if($sres != 1){
              $wpdb->query('ROLLBACK');
              throw new Exception('Can\'t delete spec');
          }
      }
      $wpdb->query('COMMIT');
      return new WP_REST_Response($data, 200);
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function delete_product($request){
    try{
        global $wpdb;
    	if(!is_numeric($request['id'])){
    		throw new InvalidArgumentException('id wrong format');
    	}
    	$id  = $request['id'];
    	$res = $wpdb->delete( $this->table, array( 'id' => $id ) );
    	if ( $res == 1 ) {
    		return new WP_REST_Response( $id, 200 );
    	}
    	throw new Exception('can\'t delete');
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

}
