<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Shipping_Controller extends WP_REST_Controller {
  private $table = 'fmwes_shipping';
  private $functions;

  public function __construct($_functions){
    $this->functions = $_functions;
  }

  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'shipping';
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::EDITABLE,
        'callback' => array($this, 'edit_shipping_list'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission_admin')
      )
    ));
    register_rest_route($namespace, '/' . $base , array(
        array(
            'methods' => WP_REST_Server::READABLE,
            'callback' => array($this, 'get_shipping'),
            'args' => array(
                'country' => array(
                    'default' => ''
                )
            ),
            'permission_callback' => array($this->functions, 'check_permission')
        )
    ));
  }
  
  public function get_shipping($request){
      try{
          global $wpdb;
          $country = sanitize_text_field($request['country']);
          $res = $wpdb->get_row( 'select * from ' . $this->table . ' where country = "' . $country .'"');
          if ( isset( $res ) ) {
              return new WP_REST_Response( $res, 200 );
          }
          else{
              throw new InvalidArgumentException('not found');
          }
          throw new Exception('can\'t get');
      }
      catch (InvalidArgumentException $iae){
          $this->functions->writeToLog($iae);
          return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
      }
      catch (Exception $e){
          $this->functions->writeToLog($e);
          return new WP_REST_Response(array('message' => $e->getMessage()), 500);
      }
  }
  
  public function edit_shipping_list($request){
      try{
          global $wpdb;
          $wpdb->query('START TRANSACTION');
          $shipping = (array) json_decode( $request->get_body() );
          $o_shipping = $wpdb->get_results('SELECT * FROM '.$this->table);
          $to_add = array();
          $to_edit = array();
          $to_delete = array();
          foreach($shipping as $ship){
              $found = false;
              foreach($o_shipping as $o_ship){
                  if($ship->country == $o_ship->country){
                      $found = true;
                      $to_edit[] = $ship;
                  }
              }
              if(!$found){
                  $to_add[] = $ship;
              }
          }
          foreach($o_shipping as $o_ship){
              $found = false;
              foreach($shipping as $ship){
                  if($o_ship->country == $ship->country){
                      $found = true;
                  }         
              }
              if(!$found){
                  $to_delete[] = $o_ship;
              }
          }
          foreach($to_add as $ship){
              $data = array(
                "country" => $ship->country,
                "checked" => $ship->checked,
                "cost" => $ship->cost
              );
              $res = $wpdb->insert($this->table, $data);
              if($res != 1){
                  $wpdb->query('ROLLBACK');
                  throw new Exception('can\'t add');
              }
          }
          foreach($to_edit as $ship){
              $data = array(
                  "country" => $ship->country,
                  "checked" => $ship->checked,
                  "cost" => $ship->cost
              );
              $res = $wpdb->update($this->table, $data, array("country" => $ship->country));
              if($res === false){
                  $wpdb->query('ROLLBACK');
                  throw new Exception('can\'t edit');
              }
          }
          foreach($to_delete as $ship){
              $res = $wpdb->delete($this->table, array("country" => $ship->country));
          }
          $wpdb->query('COMMIT');
          return new WP_REST_Response(array('message' => "ok" ), 200);
      }
      catch (InvalidArgumentException $iae){
          $this->functions->writeToLog($iae);
          return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
      }
      catch (Exception $e){
          $this->functions->writeToLog($e);
          return new WP_REST_Response(array('message' => $e->getMessage()), 500);
      }
  }
  
}