<?php

use http\Message;

/**
* @package WP_Stripe_Bancontact
*/

class Hook_Controller extends WP_REST_Controller {
  private $functions;
  private $tolerance = 300;
  private $webhook_secret_key;


  public function __construct($_functions){
    $this->functions = $_functions;
    $this->webhook_secret_key = get_option('fmwes_token');
  }

  public function register_routes(){
    $version = '1';
    $namespace = 'wp-sb/v' . $version;
    $base = 'hook';
    register_rest_route($namespace, '/' . $base, array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'check_hook'),
        'args' => array(),
        'permission_callback' => array($this, 'check_permission')
      )
    ));
    register_rest_route($namespace, '/' . $base .'/internal', array(
      array(
        'methods' => WP_REST_Server::CREATABLE,
        'callback' => array($this, 'check_hook_internal'),
        'args' => array(),
        'permission_callback' => array($this->functions, 'check_permission')
      )
    ));
  }

  public function check_hook($request){
    try{
      global $wpdb;
      $parameters = (array) json_decode( $request->get_body() );
      if($parameters['type'] == "charge.succeeded"){
        if($parameters['data']->payment_method_details->type == 'bancontact'){
          $wpdb->query('START TRANSACTION');
          $sourceid = sanitize_text_field($parameters['data']->payment_method);
          $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE sourceid="'.$sourceid.'"');
          $pi = sanitize_text_field($parameters['data']->id);
          $res = $wpdb->update('fmwes_orders', array('order_status' => 'payment_succeeded', 'payment_intent_id' => $pi), array('id' => $order->id));
          if($res != 1){
            $wpdb->query('ROLLBACK');
            throw new Exception('update order_status payment_succeeded failed');
          }
          if($order->coupon_code != null && $order->coupon_code != ""){
            $coupon = $wpdb->get_row('SELECT * FROM fmwes_coupons WHERE code="'.$order->coupon_code.'"');
            $remaining = $coupon->quantity_remaining;
            $remaining--;
            $res2 = $wpdb->update('fmwes_coupons', array('quantity_remaining' => $remaining), array('id' => $coupon->id));
            if($res2 != 1){
              $wpdb->query('ROLLBACK');
              throw new Exception('update coupon quantity_remaining failed');
            }
          }          
        }
      }else if($parameters['type'] == "charge.failed"){
        $sourceid = sanitize_text_field($parameters['data']->payment_method);
        $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE sourceid="'.$sourceid.'"');
        $res = $wpdb->update('fmwes_orders', array('order_status' => 'payment_failed'), array('id' => $order->id));
        if($res != 1){
          throw new Exception('update order_status payment_failed failed');
        }

      }else if($parameters['type'] == "source.canceled"){
          $sourceid = sanitize_text_field($parameters['data']->id);
          $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE sourceid="'.$sourceid.'"');
          $res = $wpdb->update('fmwes_orders', array('order_status' => 'payment_canceled'), array('id' => $order->id));
          if($res != 1){
              throw new Exception('update order_status payment_canceled failed');
          }
      }else if($parameters['type'] == "source.failed"){
          $sourceid = sanitize_text_field($parameters['data']->id);
          $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE sourceid="'.$sourceid.'"');
          $res = $wpdb->update('fmwes_orders', array('order_status' => 'payment_failed'), array('id' => $order->id));
          if($res != 1){
              throw new Exception('update order_status payment_failed failed');
          }
      }else if($parameters['type'] == "payment_intent.succeeded"){
        $wpdb->query('START TRANSACTION');
        $intentid = sanitize_text_field($parameters['data']->id);
        $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE payment_intent_id ="'.$intentid.'"');
        $res = $wpdb->update('fmwes_orders', array('sourceid' => $parameters['data']->payment_method, 'order_status' => 'payment_succeeded'), array('id' => $order->id));
        if($res != 1){
          $wpdb->query('ROLLBACK');
          throw new Exception('update sourceid payment_method failed');
        }
        if($order->coupon_code != null && $order->coupon_code != ""){
          $coupon = $wpdb->get_row('SELECT * FROM fmwes_coupons WHERE code="'.$order->coupon_code.'"');
          $remaining = $coupon->quantity_remaining;
          $remaining--;
          $res2 = $wpdb->update('fmwes_coupons', array('quantity_remaining' => $remaining), array('id' => $coupon->id));
          if($res2 != 1){
            $wpdb->query('ROLLBACK');
            throw new Exception('update coupon quantity_remaining failed');
          }
        }
      }else if($parameters['type'] == "account.application.deauthorized"){
          update_option('fmwes_refresh_token', '');
          update_option('fmwes_access_token', '');
      }
      $wpdb->query('COMMIT');
      if($parameters['type'] == "payment_intent.succeeded" || ($parameters['type'] == "charge.succeeded" && $parameters['data']->payment_method_details->type == 'bancontact')){
        $this->functions->send_confirmation($order);
      }
      return new WP_REST_Response("", 200);
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function check_hook_internal($request){
    try{
      global $wpdb;
      $parameters = (array) json_decode( $request->get_body() );
      if($parameters['type'] == "payment_not_needed"){
        $wpdb->query('START TRANSACTION');
        $orderid = (int)$parameters['id'];
        $order = $wpdb->get_row('SELECT * FROM fmwes_orders WHERE id ='.$orderid);
        if($order->total_price != 0){
          throw new InvalidArgumentException('Payment is needed');
          $wpdb->query('ROLLBACK');
        }
        $res = $wpdb->update('fmwes_orders', array('order_status' => 'payment_not_needed', 'sourceid' => $orderid), array('id' => $orderid));
        if($res != 1){
          $wpdb->query('ROLLBACK');
          throw new Exception('update status failed');
        }
        if($order->coupon_code != null && $order->coupon_code != ""){
          $coupon = $wpdb->get_row('SELECT * FROM fmwes_coupons WHERE code="'.$order->coupon_code.'"');
          $remaining = $coupon->quantity_remaining;
          $remaining--;
          $res2 = $wpdb->update('fmwes_coupons', array('quantity_remaining' => $remaining), array('id' => $coupon->id));
          if($res2 != 1){
            $wpdb->query('ROLLBACK');
            throw new Exception('update coupon quantity_remaining failed');
          }
        }
        $wpdb->query('COMMIT');
        return new WP_REST_Response("", 200);
      }
    }
    catch (InvalidArgumentException $iae){
			$this->functions->writeToLog($iae);
			return new WP_REST_Response(array('message' => $iae->getMessage()), 400);
		}
		catch (Exception $e){
			$this->functions->writeToLog($e);
			return new WP_REST_Response(array('message' => $e->getMessage()), 500);
		}
  }

  public function check_permission($request){
    $signature = $request->get_header('Middle-Signature');
    $signature_array = explode(",", $signature);
    foreach($signature_array as $line){
      list($k, $v) = explode('=', $line);
      $result[$k] = $v;
    }
    $signed_payload = $result['t'].'.'.str_replace("\\\\", "\\", $request->get_body());
    $myencoded = hash_hmac("sha256", $signed_payload, $this->webhook_secret_key);
    if (($this->tolerance > 0) && (\abs(\time() - $result['t']) > $this->tolerance)){
      return false;
    }
    return $myencoded == $result['v1'];
  }
}
