<?php
/**
* @package WP_Stripe_Bancontact
*/

class Database {
  public function register(){
    add_action('init', array($this, 'create_tables'));
  }


  public function create_tables(){
    global $wpdb;
    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    $sql = "CREATE TABLE fmwes_products(
      id bigint(20) NOT NULL AUTO_INCREMENT,
      title varchar(255) NOT NULL,
      description varchar(65000) NOT NULL,
      unit_price decimal(8, 2) NOT NULL,
      VAT_percentage decimal(4,2),
      image_attachment_id bigint(20),
      UNIQUE KEY id (id)) ENGINE = InnoDB;";
    maybe_create_table("fmwes_products", $sql);
    $sql = "CREATE TABLE fmwes_product_specs(
      id bigint(20) NOT NULL AUTO_INCREMENT,
      product_id bigint(20) NOT NULL,
      name varchar(255) NOT NULL,
      unit_of_measure varchar(50),
      spec_value varchar(255) NOT NULL,
      UNIQUE KEY id(id),
      CONSTRAINT product_spec_fk FOREIGN KEY (product_id) REFERENCES fmwes_products(id) ON DELETE CASCADE)ENGINE = InnoDB;";
    maybe_create_table("fmwes_product_specs", $sql);
    $sql = "CREATE TABLE fmwes_coupons(
      id bigint(20) NOT NULL AUTO_INCREMENT,
      code varchar(255) NOT NULL,
      fixed_amount decimal(8,2),
      percentage decimal(4,2),
      total_quantity int,
      quantity_remaining int,
      expiration_date date NOT NULL,
      UNIQUE KEY id(id))ENGINE = InnoDB;";
    maybe_create_table("fmwes_coupons", $sql);   
    $sql = "CREATE TABLE fmwes_orders(
      id bigint(20) NOT NULL AUTO_INCREMENT,
      timestamp datetime NOT NULL,
      total_price decimal(8,2) NOT NULL,
      order_status varchar(255) NOT NULL,
      name varchar(255) NOT NULL,
      email varchar(255) NOT NULL,
      address_line_1 varchar(255) NOT NULL,
      address_line_2 varchar(255) NOT NULL,
      zip varchar(50) NOT NULL,
      city varchar(100) NOT NULL,
      country varchar(2) NOT NULL,
      sourceid varchar(255),
      payment_intent_id varchar(255),
      type varchar(255),
      test boolean,
      coupon_code varchar(255),
      coupon_amount decimal(8,2),
      UNIQUE KEY id(id)) ENGINE = InnoDB;";
    maybe_create_table("fmwes_orders", $sql);
    $sql = "CREATE TABLE fmwes_order_products(
      order_id bigint(20) NOT NULL,
      product_id bigint(20) NOT NULL,
      quantity decimal(8, 2) NOT NULL,
      UNIQUE KEY (order_id, product_id),
      CONSTRAINT order_fk FOREIGN KEY (order_id) REFERENCES fmwes_orders(id) ON DELETE CASCADE,
      CONSTRAINT product_fk FOREIGN KEY(product_id) REFERENCES fmwes_products(id) ON DELETE CASCADE)ENGINE = InnoDB;";
    maybe_create_table("fmwes_order_products", $sql);
    $sql = "CREATE TABLE fmwes_shipping(
      country varchar(2) NOT NULL,
      cost decimal(4,2) NOT NULL,
      checked boolean
    )ENGINE = InnoDB;";
    maybe_create_table("fmwes_shipping", $sql);
    $ship_default = $wpdb->get_row('Select * from fmwes_shipping where country = "*"');
    if(!$ship_default){
        $wpdb->insert('fmwes_shipping', array(
            'country' => '*',
            'cost' => 0,
            'checked' => true
        ));
    }
    // 1.0.2
    maybe_add_column("fmwes_order_products", 'status', 'ALTER TABLE fmwes_order_products ADD status varchar(50)');
    // 2.0.0
    maybe_add_column("fmwes_product_specs", 'type', 'ALTER TABLE fmwes_product_specs ADD type varchar(50)');

  }

}
