<?php

/**
 * @package WP_Stripe_Bancontact
 */

namespace Functions {

	interface iFunctions
	{
		public function writeToLog($e);
		public function checkCoupon($code);
		public function check_permission($request);
		public function check_permission_admin($request);
		public function send_confirmation($order);
		public function get_query_from_params($params);
	}

	class Functions implements iFunctions
	{

		public function writeToLog($e)
		{
			error_log('[' . date('Y-m-d H:i:s') . '] WP Stripe Bancontact Error: ' . $e);
		}

		public function checkCoupon($code)
		{
			global $wpdb;
			if ($code == '') {
				throw new \InvalidArgumentException('coupon_code_empty');
			}
			$coupon = $wpdb->get_row('select * from fmwes_coupons where code = "' . $code . '"');
			if (!$coupon)
				throw new \InvalidArgumentException('coupon_not_found');
			if ($coupon->quantity_remaining <= 0) {
				throw new \InvalidArgumentException('coupon_used_up');
			}
			if (new \DateTime($coupon->expiration_date) < new \DateTime(date("Y-m-d"))) {
				throw new \InvalidArgumentException('coupon_expired');
			}
			return $coupon;
		}

		public function check_permission($request)
		{
			$nonce = $request->get_header('X-WP-Nonce');
			$verify = wp_verify_nonce($nonce, 'wp_rest');
			return $verify;
		}

		public function check_permission_admin($request)
		{
			$nonce = $request->get_header('X-WP-Nonce');
			$user = wp_get_current_user();
			$verify = wp_verify_nonce($nonce, 'wp_rest') && array_intersect(array('administrator'), $user->roles);
			return $verify;
		}

		public function send_confirmation($order)
		{
			global $wpdb;
			$to = $order->email;
			$subject = "";
			$subject = __("Order Confirmation From ", 'wp-stripe-bancontact') . get_bloginfo('name');
			$headers = array('Content-Type: text/html; charset=UTF-8');
			$bcc_email = get_option('fmwes_bcc_email', '');
			if ($bcc_email) {
				$headers[] = 'Bcc: ' . $bcc_email;
			}
			$products = $wpdb->get_results('SELECT p.*, op.quantity FROM `fmwes_order_products` op join fmwes_products p on p.id = op.product_id where op.order_id = ' . $order->id);
			if($products){
				$body = '<html><head></head>';
				$body .= '<body style="font-family: segoe ui; font-size: 16px; color: #555555;"><table class="main-div" style="width: 80%; background-color: #EFEFEF; margin-left:auto; margin-right:auto;">' .
					'<tr class="greeting" style="font-weight: bold;font-size: 20px; margin: 10px;"><td>' . __('Dear ', 'wp-stripe-bancontact') . $order->name . ',</td></tr>' .
					'<tr class="message-div" style="margin: 10px;"><td>' . __('We are pleased to confirm we have received your order of the following items: ', 'wp-stripe-bancontact') . '</td></tr>' .
					'<tr><td><table class="item-list" style="; margin: 10px;">';
				foreach ($products as $prod) {
					$img_src = wp_get_attachment_image_src($prod->image_attachment_id);
					$body .= '<tr style="margin: 10px;"><td>';
					$body .= $img_src ? '<img src="' . $img_src[0] . '" style="vertical-align: middle;">' : '';
					$body .= '</td>';
					$body .= '<td class="item-title" style="vertical-align: middle; font-weight: bold; margin-left: 10px;">' . $prod->title . '</td>' .
						'<td style="vertical-align: middle;"> - ' . __('Quantity', 'wp-stripe-bancontact') . ':</td>' .
						'<td class="item-quantity" style="vertical-align: middle; font-weight: bold;margin-left: 10px;">' . $prod->quantity . '</td>' .
						'</tr>';
				}
				$body .= '</table></td></tr>' .
					'<tr><td class="status-div" style="margin: 10px;"><p style="display: inline-block">' . __('Order status: ', 'wp-stripe-bancontact') . '</p>' .
					'<p class="status-result" style="font-weight: bold;display: inline-block;">' . __('Payment received', 'wp-stripe-bancontact') . '</p></td></tr>' .
					'<tr><td class="thank-you" style="margin: 10px;margin-bottom: 20px;font-weight: bold;">' . __('Thank you for your order!', 'wp-stripe-bancontact') . '</td></tr>' .
					'<tr><td style="margin: 10px; margin-bottom: 20px;">' . __("Sincerely,", 'wp-stripe-bancontact') . '</td></tr>' .
					'<tr><td style="margin: 10px; padding-bottom: 20px;">' . get_bloginfo('name') . '</td></tr>' .
					'<tr><td style="font-style: italic; font-size: 12px;">' . __('powered by ') . '<a href="https://five-minute-webshop.com">Five Minute Webshop</a></td></tr>' .
					'</table></body></html>';


				wp_mail($to, $subject, $body, $headers);
			}
		}

		public function get_query_from_params($params)
		{
			global $wpdb;
			$where = ' where ';
			foreach ($params as $key => $value) {
				// check spec type
				$spec = $wpdb->get_var("SELECT type from fmwes_product_specs where name = '" . $key . "'");
				if ($spec == 'numeric') {
					$range = explode('-', $value);
					if (count($range) != 2) {
						throw new \InvalidArgumentException('parameter of wrong format');
					}
					$where .= "AND name = '" . $key . "' AND CAST(ps.spec_value as decimal(20,4)) BETWEEN " . (float)$range[0] . " and " . (float)$range[1] . " ";
				} else if ($spec == 'string') {
					$where .= "AND ps.spec_value = " . $value . " ";
				}
			}
			// remove leading AND
			$where = preg_replace("AND", "", $where, 1);
		}
	}
}