<?php
/**
 * @package WP_Stripe_Bancontact
 */

interface iHttpFunctions{
  public function makeHttpRequest($type, $uri, $body=null);
  public function getAccountId();
}

use \Firebase\JWT\JWT;

class HttpFunctions implements iHttpFunctions{

  public function makeHttpRequest($type, $uri, $body=null){
    $access_token = get_option('fmwes_access_token');
    $headers = array(
      'authorization' => 'bearer ' . $access_token,
      'Content-Type' => 'application\json'
    );
    $mrequest = new WP_Http;
    $result = $mrequest->request($uri, array('timeout' => 10, 'method' => $type, 'headers' => $headers, 'body' => json_encode($body)));
    if($result['response']['code'] == 401){
      $refresh_token = get_option('fmwes_refresh_token');
      $r_mrequest = new WP_Http;
      $r_headers = array(
        'authorization' => 'bearer ' . $refresh_token
      );
      $r_result = $r_mrequest->request(FMWES_MIDDLE_URL . '/access', array('method' => 'POST', 'headers' => $r_headers));
      if($r_result['response']['code'] == 200){
        $tokens = json_decode($r_result['body']);
        if($tokens->access_token && $tokens->refresh_token){
            update_option('fmwes_access_token', $tokens->access_token);
            update_option('fmwes_refresh_token', $tokens->refresh_token);
        }
        $nheaders = array(
          'authorization' => 'bearer ' . $tokens->access_token,
          'Content-Type' => 'application\json'
        );
        $nrequest = new WP_Http;
        $nresult = $nrequest->request($uri, array('method' => $type, 'headers' => $nheaders, 'body' => json_encode($body)));
        return $nresult;
      }
      else{
        return $r_result;
      }
    }
    return $result;
  }

  public function getAccountId(){
    $token = get_option('fmwes_refresh_token');
    $decoded = JWT::decode($token, FMWES_M_PUB_KEY, array('RS256'));
    return $decoded->account_id;
  }

}
